#if !defined(__sys_iop_h)
#define __sys_iop_h

#if !defined(_NO_IDENTS) && defined(_HEAD_IDENTS)
# pragma ident "@(#)head:sys/iop.h	1.4 91/07/10 {Apple version 3.0 90/11/29 11:37:56}"
#endif

/*
 * Copyright 1987, 1988, 1989 Apple Computer, Inc.
 * All Rights Reserved.
 *
 * THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF APPLE COMPUTER, INC.
 * The copyright notice above does not evidence any actual or
 * intended publication of such source code.
 */

#ifndef u_char
#if defined(__MPW_C__)
# include "/:usr:include:sys:types.h"
#else
# include <sys/types.h>
#endif
#endif

/* I/O Processor (IOP) Control registers.  These are at addresses which
 * are double those in the IOP document since the 68K doesn't have an
 * A0 bit...
 *
 * NB: The Mac ROM, since it doesn't use C structures, can access
 * "ramaddr" as a WORD at offset 01, but it's misaligned for C code...
 */

struct iop {
    u_char	ramaddrH;	/* 00: shared RAM address hi byte */
    u_char	junk0;		/* 01: unused */
    u_char	ramaddrL;	/* 02: shared RAM address lo byte */
    u_char	junk1;		/* 03: unused */
    u_char	statctl;	/* 04: status/control register */
    u_char	junk2[3];	/* 05-07: unused */
    u_char	ramdata;	/* 08: RAM data byte at ramhi/lo */

    /* Bypass-mode hardware access from the host: */

    u_char	junk3[23];	/* 09-1f: unused */
    union {
	struct {		/* SCC registers */
	    u_char b_cmd;	/* 20: SCC B command reg */
	    u_char junk4;
	    u_char a_cmd;	/* 22: SCC A command reg */
	    u_char junk5;
	    u_char b_data;	/* 24: SCC B data */
	    u_char junk6;
	    u_char a_data;	/* 26: SCC A data */
	} scc;

	struct {		/* ISM registers */
	    u_char wdata;	/* 20: write a data byte */
	    u_char junk7;
	    u_char wmark;	/* 22: write a mark byte */
	    u_char junk8;
	    u_char wcrc;	/* 24: write 2-byte crc to disk */
	    u_char junk9;
	    u_char wparams;	/* 26: write the param regs */
	    u_char junk10;
	    u_char wphase;	/* 28: write the phase states & dirs */
	    u_char junk11;
	    u_char wsetup;	/* 2a: write the setup register */
	    u_char junk12;
	    u_char wzeroes;	/* 2c: mode reg: 1's clr bits, 0's are x */
	    u_char junk13;
	    u_char wones;	/* 2e: mode reg: 1's set bits, 0's are x */
	    u_char junk14;
	    u_char rdata;	/* 30: read a data byte */
	    u_char junk15;
	    u_char rmark;	/* 32: read a mark byte */
	    u_char junk16;
	    u_char rerror;	/* 34: read the error register */
	    u_char junk17;
	    u_char rparams;	/* 36: read the param regs */
	    u_char junk18;
	    u_char rphase;	/* 38: read the phase states & dirs */
	    u_char junk19;
	    u_char rsetup;	/* 3a: read the setup register */
	    u_char junk20;
	    u_char rmode;	/* 3c: read the mode register */
	    u_char junk21;
	    u_char rhandshake;	/* 3e: read the handshake register */
	} ism;
    } b;
};

/* IOP status/control register bits: */

#define	IOP_BYPASS	0x01	/* bypass-mode hardware access */
#define	IOP_AUTOINC	0x02	/* allow autoincrement of ramhi/lo */
#define	IOP_RUN		0x04	/* set to 0 to reset IOP chip */
#define	IOP_IRQ		0x08	/* generate IRQ to IOP if 1 */
#define	IOP_INT0	0x10	/* intr priority from IOP to host */
#define	IOP_INT1	0x20	/* intr priority from IOP to host */
#define	IOP_HWINT	0x40	/* IRQ from hardware; bypass mode only */
#define	IOP_DMAINACTIVE	0x80	/* no DMA request active; bypass mode only */

/* Bit combinations which are written to change things: */

#define	IOP_CLRINT0	(IOP_AUTOINC | IOP_RUN | IOP_INT0                     )
#define	IOP_CLRINT1	(IOP_AUTOINC | IOP_RUN |            IOP_INT1          )
#define	IOP_STOPIOP	(IOP_AUTOINC	           			      )
#define	IOP_START	(IOP_AUTOINC | IOP_RUN				      )
#define	IOP_INTERRUPT	(IOP_AUTOINC | IOP_RUN |		       IOP_IRQ)
#define	IOP_INCENABLE	(IOP_AUTOINC | IOP_RUN                                )
#define	IOP_INCDISABLE	(              IOP_RUN                                )

#endif /* __sys_iop_h */
