#if !defined(__sys_iopkernel_h)
#define __sys_iopkernel_h

#if !defined(_NO_IDENTS) && defined(_HEAD_IDENTS)
# pragma ident "@(#)head:sys/iopkernel.h	1.4 91/07/10 {Apple version 3.0 90/11/29 11:37:56}"
#endif

/*
 * Copyright 1987, 1988, 1989 Apple Computer, Inc.
 * All Rights Reserved.
 *
 * THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF APPLE COMPUTER, INC.
 * The copyright notice above does not evidence any actual or
 * intended publication of such source code.
 */

#ifndef u_char
#if defined(__MPW_C__)
# include "/:usr:include:sys:types.h"
#else
# include <sys/types.h>
#endif
#endif

/* I/O Processor (IOP) Kernel control info */

#define	MSG_IDLE	0	/* idle */
#define	MSG_NEWSENT	1	/* new msg sent */
#define	MSG_RCVD	2	/* msg received, being processed */
#define	MSG_COMPLETE	3	/* msg processing complete, reply available */

/* IOP to CPU Message type codes: */
#define	TYP_MOVE	1	/* data move */

#define NCH 7

struct iopkernel {
    u_char	junk1[0x200];	/* 0000-01ff: not applicable */

    u_char	maxsend;	/* 0200: max msg number used by this IOP */
    u_char	sendstate[NCH];	/* 0201-0207: states for msgs 1-7 */
    u_char	junk2[23];	/* 0208-021e : unused */
    u_char	patchflag;	/* 021f: patch control flag */
    u_char	send[32 * NCH];	/* 0220-02ff: messages 1-7 */

    u_char	maxrcv;		/* 0300: max msg number used by this IOP */
    u_char	rcvstate[NCH];	/* 0301-0307: states for msgs 1-7 */
    u_char	junk3[23];	/* 0308-031e : unused */
    u_char	alive;		/* 031f: changed to 0xff if IOP alive */
    u_char	rcv[32 * NCH];	/* 0320-03ff: messages 1-7 */
};

/* Hard defines for those who wish to use them. */

#define	IOP_MAXSEND	0x200
#define	IOP_SENDSTATE	0x201 - 1	/* for zero-based channels */
#define	IOP_PATCHFLAG	0x21f
#define	IOP_SEND	0x220 - 32	/* for zero-based channels */
#define	IOP_MAXRCV	0x300
#define	IOP_RCVSTATE	0x301 - 1	/* for zero-based channels */
#define	IOP_ALIVE	0x31f
#define	IOP_RCV		0x320 - 32	/* for zero-based channels */

/* Notes:

 * To patch IOP code, we set "patchflag" to MSG_NEWSENT.  The IOP
 * will disable its interrupts, change patchflag to MSG_COMPLETE, and
 * then wait in an interrupt-disabled loop while we patch its code.
 * When we're finished mucking around, we set patchflag to MSG_IDLE.
 * The IOP enables interrupts and continues.
 *
 * We should check "alive" periodically, making sure it is set to
 * 0xff by the IOP, then resetting it to something like zeroes.  The
 * IOP will keep stuffing 0xff back in alive to prove it's alive.
 *
 * EXAMPLE, probably in some timer-driven piece of the kernel:
 *	if (alive != 0xff) {
 *		complain("iop is dead");
 *	} else {
 *		alive = 0;
 *	}
 */

#endif /* __sys_iopkernel_h */
