#if !defined(__sys_iopmgr_h)
#define __sys_iopmgr_h

#if !defined(_NO_IDENTS) && defined(_HEAD_IDENTS)
# pragma ident "@(#)head:sys/iopmgr.h	1.8 91/09/19 {Apple version 3.0 90/11/29 11:37:56}"
#endif

/*
 * Copyright 1987-91 Apple Computer, Inc.
 * All Rights Reserved.
 *
 * THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF APPLE COMPUTER, INC.
 * The copyright notice above does not evidence any actual or
 * intended publication of such source code.
 */

#if !defined(__sys_types_h)
# if defined(__MPW_C__)
#   include "/:usr:include:sys:types.h"
# else
#   include <sys/types.h>
# endif
#endif 

#define	SPLIOP	spl5

/* Move info */

#define	MV_TO_CPU	0	/* move data from IOP to CPU */
#define	MV_TO_IOP	1	/* move data from CPU to IOP */
#define	MV_COMPARE	2	/* compare memory CPU vs IOP */
#define	MV_PATCH	3	/* patch the IOP */

struct movemsg {		/* info for data moves */
    char	type;		/* move type */
    char	iop;		/* which IOP */
    u_short	count;		/* number of bytes (exc. for patch) */
    union {
	caddr_t	buf;		/* main CPU memory data buffer */
	struct codepacket *pkt;	/* codepacket list for patches */
    } b;
    u_short	offset;		/* where in IOP memory */
    char	compresult;	/* -1: IOP<CPU; +1: IOP>CPU; 0: equal */
    u_char	junk1;		/* reserved */
};

struct codepacket {		/* patch info */
    u_char	count;		/* how many bytes, 0-255 */
    u_short	offset;		/* where in IOP memory */
    u_char	code[1];	/* the code follows here */
};

/* Request codes: */
#define	IOPCMD_SEND		1	/* send a message */
#define	IOPCMD_MOVE		2	/* move data to/from IOP */
#define	IOPCMD_ADDRCVR		3	/* install a receive-message handler */
#define	IOPCMD_DELRCVR		4	/* delete  a receive-message handler */
#define	IOPCMD_INTOBYPASS	5	/* put  IOP into bypass mode */
#define	IOPCMD_NOBYPASS		6	/* take IOP out of bypass mode */
#define	IOPCMD_RCVREPLY		7	/* send reply (and MSG_COMPLETE) */
#define	IOPCMD_STARTIOP		8	/* start an IOP (after download) */
#define	IOPCMD_STOPIOP		9	/* stop  an IOP (for   download) */
#define	IOPCMD_CLRMEM		10	/* clear entire IOP memory */

#define	IOPCMD_MAX		IOPCMD_CLRMEM

#define	MSGLEN 32		/* max message length */

struct iopreq {			/* request to IOP manager */
    struct iopreq *link;	/* forward queue link */
    struct movemsg move;	/* info for a data-move request */
    short	req;		/* request type */
    short	flags;		/* flags */
    short	iop;		/* which IOP */
    short	chan;		/* channel number */
    short	msglen;		/* message length */
    short	replylen;	/* reply length */
    caddr_t 	msg;		/* message to send */
    caddr_t 	reply;		/* reply message buffer */
    int		(*handler)();	/* reply message handler function */
    u_long	driver;		/* for use by caller, untouched */
    u_char	sync;		/* synchronous write requested */
};

/* Flags: */
#define	REQ_BUSY	0x0001	/* request is busy (running or queued) */
#define	REQ_QUEUED	0x0002	/* request is in send queue */
#define	REQ_RCVR	0x0004	/* request is a receive-message receiver */

/* Error codes (zero always means "no error" : */
#define	EIOPNOTINIT	1	/* IOP not initialized */
#define	EIOPBYPASS	2	/* IOP in bypass mode, func not applicable */
#define	EIOPHASRCVR	3	/* receive-msg handler already installed */
#define	EIOPNORCVR	4	/* no receive-msg handler to delete */
#define	EIOPWRONGRCVR	5	/* tried to remove other receiver */
#define	EIOPCHANRANGE	6	/* tried to send to invalid channel number */
#define	EIOPDEAD	7	/* IOP is dead */
#define	EIOPCMD		8	/* invalid request */
#define	EIOPINUSE	9	/* IOP in use, can't do this command! */
#define	EIOPMVCMD	10	/* invalid move request */
#define	EIOPREQBUSY	11	/* req.link or flags nonzero: request in use? */

/* Iop Manager NOTES:
 *
 * 1. A receive-message handler installed by IOP_ADDRCVR will remain in
 *    effect until removed by IOP_DELRCVR.
 *
 * 2. It is permissible to omit (i.e. 0) the  "handler" function address
 *    on an IOPCMD_SEND or IOPCMD_ADDRCVR.  For IOPCMD_SEND, this means
 *    that no call will be made when the "message complete" status comes
 *    back from the IOP.  For IOPCMD_ADDRCVR, the Iop Manager will simply
 *    dump the incoming message on the console and acknowledge it to
 *    the IOP.
 *
 *    Neither is very useful, except maybe in the testing phase.
 *
 * 3. An IOP cannot be put into bypass-mode if there are outstanding
 *    requests or there are receive-message handlers installed.
 *
 * 4. The following commands ALWAYS complete immediately, without ever
 *    calling req.handler:
 *	IOPCMD_MOVE
 *	IOPCMD_ADDRCVR
 *	IOPCMD_DELRCVR
 *	IOPCMD_INTOBYPASS
 *	IOPCMD_NOBYPASS
 *	IOPCMD_RCVREPLY
 *	IOPCMD_STARTIOP
 *	IOPCMD_STOPIOP
 *	IOPCMD_CLRMEM
 *
 *    Basically, the ONLY time req.handler will be called is after
 *    a "message complete" on an IOPCMD_SEND message, and when an
 *    incoming message is passed to a receive-message handler installed
 *    via IOPCMD_ADDRCVR.
 */

#endif /* __sys_iopmgr_h */
