#if !defined(__sys_scciop_h)
#define __sys_scciop_h

#if !defined(_NO_IDENTS) && defined(_HEAD_IDENTS)
# pragma ident "@(#)head:sys/scciop.h	1.8 91/09/19 {Apple version 3.0 90/11/29 11:37:56}"
#endif

/*
 * Copyright 1987-91 Apple Computer, Inc.
 * All Rights Reserved.
 *
 * THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF APPLE COMPUTER, INC.
 * The copyright notice above does not evidence any actual or
 * intended publication of such source code.
 */

#if !defined(__sys_types_h)
# if defined(__MPW_C__)
#   include "/:usr:include:sys:types.h"
# else
#   include <sys/types.h>
# endif
#endif 

/* SCC I/O Processor Interface */

/* IOP and channel values: */
#define	SCC_IOP		0	/* SCC IOP */

#define	DVR_A		0	/* port A driver */
#define	DVR_B		1	/* port B driver */
#define	DVR_KERNEL	2	/* the IOP kernel */

/* Channel assignments: */

/* Serial IOP kernel messages: */
#define	SCC_KERN_MSG	1	/* host-to-IOP */

/* Port A (modem) */
#define	SCCA_READ	2
#define	SCCA_WRITE	3
#define	SCCA_CTL	4
#define	SCCA_EVENT	3	/* unsolicited IOP-to-host */

/* Port B (printer) */
#define	SCCB_READ	5
#define	SCCB_WRITE	6
#define	SCCB_CTL	7
#define	SCCB_EVENT	6	/* unsolicited IOP-to-host */

/* SERIAL IOP KERNEL MESSAGES AND CODES: */

#define kern_alloc	1	/*   allocate driver */
#define	kern_dealloc	2	/* deallocate driver */
#define	kern_initdvr	3	/* initialize driver */
#define	kern_bypass	4	/* go in/out of bypass mode */
#define	kern_dversion	5	/* get driver version info */

#define	Enotalloc	-6	/* driver not allocated */
#define	Einbypass	-5	/* IOP is in bypass mode */
#define	Edvrinuse	-4	/* driver is in use */
#define	Eerror		-1	/* other error */

#define	KERNBASE	0x0400	/* base of the IOP kernel */
#define	DVRABASE	0x2d55	/* base of driver A */
#define	DVRBBASE	0x56aa	/* base of driver B */

struct kmsg_alloc {
    u_char	code;		/* command code 0x01 */
    u_char	driver;		/* driver: 00=A, 01=B */
    u_char	clientid;	/* our nonzero client ID */
};

struct kreply_alloc {		/* reply to kmsg_alloc */
    u_char	result;		/* error code */
    u_char	clientid;	/* client ID if err=Edvrinuse or Einbypass */
    u_short	addr;		/* driver start address */
};

struct kmsg_dealloc {
    u_char	code;		/* command code 0x02 */
    u_char	driver;		/* driver: 00=A, 01=B */
};

struct kreply_dealloc {		/* reply to kmsg_dealloc */
    u_char	result;		/* error code */
    u_char	clientid;	/* client ID if err=Einbypass */
};

struct kmsg_initdvr {
    u_char	code;		/* command code 0x03 */
    u_char	driver;		/* driver: 00=A, 01=B */
};

struct kreply_initdvr {		/* reply to kmsg_initdvr */
    u_char	result;		/* error code */
    u_char	clientid;	/* client ID if err=Einbypass */
};

struct kmsg_bypass {
    u_char	code;		/* command code 0x04 */
    u_char	onoff;		/* 0=off, 1=on */
    u_char	clientid;	/* our nonzero client ID */
};

struct kreply_bypass {		/* reply to kmsg_bypass */
    u_char	result;		/* error code */
/*  union { */
    u_char	bypassid;	/* client ID if in bypass */
    u_char	dvrAid;		/* client ID if driver A allocated */
/*  } dvrAid; */
    u_char	dvrBid;		/* client ID if driver B allocated */
    /****** NB: I've asked Bill O'Connor to dump the union altogether!  **/
    /* But it might not hapen....XXX */
};

struct kmsg_version {
    u_char	code;		/* command code 0x05 */
    u_char	driver;		/* driver: 00=A, 01=B, 02=kernel */
};

struct kreply_version {		/* reply to kmsg_version */
    u_char	result;		/* error code */
    /* NB I've assumed Bill O'connor will allow the pad here. */
    /* maybe he can return a length???  XXX */
    u_char	pad;
    u_short	info;		/* addr of version info in IOP memory */
};

#define	VERLEN	128		/* max size of version string we support */

/* IOP SERIAL DRIVER MESSAGES AND CODES: */

#define	cc_control	0x00	/* control */
#define	cc_open		0x01	/* open */
#define	cc_close	0x02	/* close */

/* Control operations: */

#define	ctl_reset	0x08	/* serial reset */
#define	ctl_handshake	0x0a	/* serial handshake */
#define	ctl_clearbreak	0x0b	/* clear break mode */
#define	ctl_setbreak	0x0c	/* set break mode */
#define	ctl_setbaud	0x0d	/* set baud rate */
#define	ctl_extshake	0x0e	/* extended serial handshake */
#define	ctl_options	0x10	/* options */
#define	ctl_assertdtr	0x11	/* assert DTR */
#define	ctl_cleardtr	0x12	/* clear DTR */
#define	ctl_setPEsub	0x13	/* set PE substitution char */
#define	ctl_setaltPEsub	0x14	/* set alternate PE sub char */
#define	ctl_setxoff	0x15	/* set XOFF */
#define	ctl_clearxoff	0x16	/* clear XOFF */
#define	ctl_sendxon	0x17	/* send XON if XOFF'ed */
#define	ctl_usendxon	0x18	/* unconditionally send XON */
#define	ctl_sendxoff	0x19	/* send XOFF if XON'ed */
#define	ctl_usendxoff	0x1a	/* unconditionally send XOFF */
#define	ctl_resetSCC	0x1b	/* reset SCC */

/* The serial configuration word format: */

struct cnfg {
    u_short stopbits:2;		/* stop bits */
    u_short parity:2;		/* parity type */
    u_short charbits:2;		/* bits per character */
    u_short divisor:10;		/* "old" Mac baud rate divisor */
};
#define	cnf_1stop	1	/* 1   stop bit */
#define	cnf_15stop	2	/* 1.5 stop bits */
#define	cnf_2stop	3	/* 2   stop bits */
#define cnf_noparity	0	/* no   parity */
#define cnf_oddparity	1	/* odd  parity */
#define cnf_noparity2	2	/* no   parity */
#define cnf_evenparity	3	/* even parity */
#define cnf_5bits	0	/* 5 bits/char */
#define cnf_7bits	1	/* 7 bits/char */
#define cnf_6bits	2	/* 6 bits/char */
#define cnf_8bits	3	/* 8 bits/char */

/* The serial status information, found at "statbuf+1" from open call.
 * NB: If the u_char at "statbuf" is nonzero, then "cumErrs" should
 * be zeroed.  After reading the status bytes, the driver should
 * write 0xff into statbuf+0 to cause it to be updated.
 */

struct serstatus {
    u_char	cumErrs;	/* cumulative errors */
    u_char	xOffSent;	/* 0x80=input flow off by XOFF, 0x40 by DTR */
    u_char	rdPend;		/* 0xff = read  command pending */
    u_char	wrPend;		/* 0xff = write command pending */
    u_char	ctsHold;	/* 0x00 = clear to send */
    u_char	xOffHold;	/* 0xff = XOFF received */
};

/* cumErrs bits: */
#define	Eoverflow	0x01	/* input buffer overflow */
#define	Ebreak		0x08	/* break received */
#define	Eparity		0x10	/* parity error */
#define	Eoverrun	0x20	/* hardware overrun */
#define	Eframing	0x40	/* framing error */

/* xOffSent bits: */
#define	offbyx		0x80	/* output flow stopped by received XOFF */
#define	offbydtr	0x40	/* output flow stopped by dropping DTR */

struct cmd_generic {		/* used only to get result from any cmd */
    u_char	result;		/* (out) result code */
    u_char	code;		/* control code 0x00 */
    u_char	op;		/* control op */
};

struct cmd_open {
    u_char	result;		/* (out) result code */
    u_char	code;		/* cmd code 0x01 */
    u_short	junk1;		/* unused */
    struct cnfg	config;		/* serial configuration word */
    u_short	divisor;	/* baud rate divisor */
    u_short	writebuf;	/* (out) write buffer address */
    short	wbuflen;	/* (out) write buffer length */
    u_short	statbuf;	/* (out) stats buffer address */
};
/* NB: "new" divisor may be calculated from config.divisor as follows:
 *
 *  divisor = ((cnfg.divisor + 2) * 36864 / 36707) - 2
 *
 * Otherwise, just use: divisor = (115200 / baudrate) - 2
 */

struct cmd_read {
    u_char	result;		/* (out) result code */
    u_char	junk1;
    u_char	junk2;
    u_char	junk3;
    short	taken;		/* bytes taken from ring buffer */
    union {
	u_char	abortack;	/* nonzero to acknowledge the abort */
	u_short	ringbuf;	/* (out) address of ring buffer */
    } u;
    u_short	ringend;	/* (out) address of ringbufend + 1 */
    u_short	first;		/* (out) address of first byte of ring buffer */
    short	buflen;		/* (out) # of bytes in ring buffer */
    short	abortcnt;	/* (out) # of bytes before abort occurred */
};

struct cmd_write {
    u_char	result;		/* (out) result code */
    u_char	junk1;
    u_char	junk2;
    u_char	junk3;
    short	count;		/* length of data to write */
};

struct cmd_close {
    u_char	result;		/* (out) result code */
    u_char	code;		/* cmd code 0x02 */
};

struct cmd_reset {
    u_char	result;		/* (out) result code */
    u_char	code;		/* control cmd code 0x00 */
    u_char	op;		/* control opcode 0x08 */
    u_char	junk1;
    struct cnfg	config;		/* serial configuration word */
    u_short	divisor;	/* baud rate divisor */
};

struct cmd_handshake {
    u_char	result;		/* (out) result code */
    u_char	code;		/* control cmd code 0x00 */
    u_char	op;		/* control opcode 0x0a */
    u_char	junk1;
    u_char	outxon;		/* nonzero enables XON/XOFF output flow ctl */
    u_char	outcts;		/* nonzero enables CTS      output flow ctl */
    u_char	xonchar;	/* XON  character */
    u_char	xoffchar;	/* XOFF character */
    u_char	errabort;	/* errors that cause abort */
    u_char	statevent;	/* status changes that cause events */
    u_char	inxon;		/* nonzero enables XON/XOFF input flow ctl */
};

/* Bit masks for "errabort" and "statevent" bytes: */

#define	ABT_FRAMING	0x40	/* abort on framing error */
#define	ABT_OVERRUN	0x20	/* abort on overrun error */
#define	ABT_PARITY	0x10	/* abort on parity  error */

#define	POST_BREAK	0x80	/* post event on break change */
#define	POST_CTS	0x20	/* post event on CTS   change */

struct cmd_clearbreak {
    u_char	result;		/* (out) result code */
    u_char	code;		/* control cmd code 0x00 */
    u_char	op;		/* control opcode 0x0b */
};

struct cmd_setbreak {
    u_char	result;		/* (out) result code */
    u_char	code;		/* control cmd code 0x00 */
    u_char	op;		/* control opcode 0x0c */
};

struct cmd_setbaud {
    u_char	result;		/* (out) result code */
    u_char	code;		/* control cmd code 0x00 */
    u_char	op;		/* control opcode 0x0d */
    u_char	junk1;
    u_short	divisor;	/* baud rate divisor (115200 / baud) - 2 */
};

struct cmd_extshake {
    u_char	result;		/* (out) result code */
    u_char	code;		/* control cmd code 0x00 */
    u_char	op;		/* control opcode 0x0e */
    u_char	junk1;
    u_char	outxon;		/* nonzero enables XON/XOFF output flow ctl */
    u_char	outcts;		/* nonzero enables CTS      output flow ctl */
    u_char	xonchar;	/* XON  character */
    u_char	xoffchar;	/* XOFF character */
    u_char	errabort;	/* errors that cause abort */
    u_char	statevent;	/* status changes that cause events */
    u_char	inxon;		/* nonzero enables XON/XOFF input flow ctl */
    u_char	indtr;		/* nonzero enables DTR      input flow ctl */
};

struct cmd_options {
    u_char	result;		/* (out) result code */
    u_char	code;		/* control cmd code 0x00 */
    u_char	op;		/* control opcode 0x10 */
    u_char	junk1;
    u_char	options;	/* options byte */
};
#define	CTL_LEAVEDTR	0x80	/* leave DTR unchanged at close */
#define	CTL_SETEXT	0x40	/* set external clocking */

struct cmd_assertdtr {
    u_char	result;		/* (out) result code */
    u_char	code;		/* control cmd code 0x00 */
    u_char	op;		/* control opcode 0x11 */
};

struct cmd_cleardtr {
    u_char	result;		/* (out) result code */
    u_char	code;		/* control cmd code 0x00 */
    u_char	op;		/* control opcode 0x12 */
};

struct cmd_setPEsub {
    u_char	result;		/* (out) result code */
    u_char	code;		/* control cmd code 0x00 */
    u_char	op;		/* control opcode 0x13 */
    u_char	junk1;
    u_char	sub;		/* PE substitution character */
};

struct cmd_setaltPEsub {
    u_char	result;		/* (out) result code */
    u_char	code;		/* control cmd code 0x00 */
    u_char	op;		/* control opcode 0x14 */
    u_char	junk1;
    u_char	sub;		/* PE substitution character */
    u_char	altsub;		/* alternate PE substitution character */
};

struct cmd_setxoff {
    u_char	result;		/* (out) result code */
    u_char	code;		/* control cmd code 0x00 */
    u_char	op;		/* control opcode 0x15 */
};

struct cmd_clearxoff {
    u_char	result;		/* (out) result code */
    u_char	code;		/* control cmd code 0x00 */
    u_char	op;		/* control opcode 0x16 */
};

struct cmd_sendxon {
    u_char	result;		/* (out) result code */
    u_char	code;		/* control cmd code 0x00 */
    u_char	op;		/* control opcode 0x17 */
};

struct cmd_usendxon {
    u_char	result;		/* (out) result code */
    u_char	code;		/* control cmd code 0x00 */
    u_char	op;		/* control opcode 0x18 */
};

struct cmd_sendxoff {
    u_char	result;		/* (out) result code */
    u_char	code;		/* control cmd code 0x00 */
    u_char	op;		/* control opcode 0x19 */
};

struct cmd_usendxoff {
    u_char	result;		/* (out) result code */
    u_char	code;		/* control cmd code 0x00 */
    u_char	op;		/* control opcode 0x1a */
};

struct cmd_resetSCC {
    u_char	result;		/* (out) result code */
    u_char	code;		/* control cmd code 0x00 */
    u_char	op;		/* control opcode 0x1b */
};

struct sccevent {		/* SCC event message */
    u_long	junk1;		/* unused */
    u_char	reg0;		/* (out) SCC read-reg 0 at interrupt time */
    u_char	changed;	/* (out) SCC read-reg 0 bits that've changed */
};

#endif /* __sys_scciop_h */
