#if !defined(__sys_scsireq_h)
#define __sys_scsireq_h

#if !defined(_NO_IDENTS) && defined(_HEAD_IDENTS)
# pragma ident "@(#)head:sys/scsireq.h	1.11 92/09/09 {Apple version 3.0 90/11/29 11:37:56}"
#endif

/*
 * Copyright 1987-91 Apple Computer, Inc.
 * All Rights Reserved.
 *
 * THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF APPLE COMPUTER, INC.
 * The copyright notice above does not evidence any actual or
 * intended publication of such source code.
 */

#if !defined(__sys_types_h)
# if defined(__MPW_C__)
#   include "/:usr:include:sys:types.h"
# else
#   include <sys/types.h>
# endif
#endif 

#if !defined(__sys_vio_h)
# if defined(__MPW_C__)
#   include "/:usr:include:sys:vio.h"
# else
#   include <sys/vio.h>
# endif
#endif 

/*
 *      This file contains definitions for interfacing drivers with the
 *	A/UX Scsi Manager.  First the request struct is defined, along
 *	with its multitudinous control flags.  Then the return codes
 *	are defined as well as external printable diagnostic strings.
 */

/*****
 *
 *	scsireq ( id, scsireq, viop) - enter scsi request
 *	scsirequest (id, scsireq)
 *		Function scsireq schedules a scsi request for running
 *		and returns immediately; also passes scatter-gather
 *		vio struct data.  Second form same as first, but
 *		without vio struct; (vio will be built from contained
 *		databuf/datalen)
 *
 *      The "scsireq" struct below packages all necessary data
 *	to describe a complete Initiator/Target transaction.
 *      Request flags are defined below, as are the various Manager
 *	return codes.
 *
 */
int	scsireq();

#define SPLSCSI spl2		/* must be >= builtin and slot scsi devs */

/* for compatibility, map 1.0 call-> 1.1 */
#define scsirequest(i,r) scsireq((i),(r),(struct vio *)0)

#define REVCODE "1.1"		/* 3 char rev code, 1st 2 must match. */
				/* set or init into revcode below */

struct	scsireq {
	caddr_t cmdbuf;		/* buffer containing command block */
	caddr_t	databuf;	/* buffer containing the data to send or rcv */
	unsigned datalen;	/* length of the data buffer */
	unsigned datasent;	/* length of data actually sent or received */
	caddr_t	sensebuf;	/* optional error result from sense cmd */
	int	(*faddr)();	/* address of completion function */
	long	driver;		/* private storage for driver placing request */
	struct	scsireq *link;	/* link to next request */
	u_short	flags;		/* request flag bits */
	u_char	cmdlen;		/* length of command buffer */
	u_char	senselen;	/* optional length of the sense buffer */
	u_char	sensesent;	/* length of sense data recieved */
	u_char	msg;		/* mode byte returned on cmd complete */
	u_char	stat;		/* completion status byte */
	u_char	ret;		/* return code from UNIX SCSI software */
	u_char  reserve1;	/* reserved */
	short	timeout;	/* maximum seconds inactivity for request */
	char    revcode[4];	/* 3-char scsi manager version, strncpy REVCODE */
	struct  vio vio;	/* acceleration scatter-gather io dma */
	int     niovsent;	/* number of iov-s actually sent */
	};


/*	Request flag bits.
 */

#define	SRQ_READ     0x01 	/* This request reads data */
#define	SRQ_EXCL     0x02	/* This request must have exclusive use */
#define SRQ_TRACE    0x04	/* Record           bus trace this req */
#define SRQ_TRACEPR  0x08	/* Record and print bus trace this req */
#define SRQ_PRISHIFT 4		/* left shift by this for priority field */
#define	SRQ_PRIMASK  0x30	/* Four Priority levels */
#define SRQ_PRI0     0x00
#define SRQ_PRI1     0x10
#define SRQ_PRI2     0x20
#define SRQ_PRI3     0x30

#define SRQ_NOINTR   0x40	/* Run this request at spl7 */
#define SRQ_SPLINT   0x80	/* Run this request at spl2 */

#define	SRT_NOTIME    -1	/* No time out for this request */



/*
 *	SCSI MANAGER return codes from scsireq().
 *	Device drivers should cope easily with errors > SST_FATAL
 *	error codes < SST_FATAL may indicate problems with the SCSI bus,
 *	and probably should be attempted again.
 */

#define	SST_BSY     1	/* Device dropped busy unexpectedly */
#define	SST_CMD	    2	/* An error happened during command transmission */
#define	SST_COMP    3	/* an error occured during the status phase */
#define SST_SENSE   4	/* An error occured obtaining sense data */
#define	SST_SEL	    5	/* No unit reponded to the given id selection */
#define	SST_TIMEOUT 6	/* Request was idle longer than timeout requested */
#define	SST_MULT    7	/* Driver(s) placed multiple requests for this id */
#define	SST_PROT    8	/* A problem occured in SCSI protocol */
#define	SST_MORE    9	/* There was more data than the device expected */
#define	SST_LESS    10	/* There was less data than the device expected */
#define	SST_FATAL   8
#define	SST_STAT    11	/* Status word was non zero, sense command is set up */
#define	SST_AGAIN   12	/* A transfer error occured, place request again */
#define	SST_REVCODE 13	/* Caller or Manager out-of-date, must recompile */
#define	SST_CANCEL 14	/* Request cancelled by call to scsicancel  */


/*****	scsistrings -- error message strings.
 *
 *	This global data structure is indexed by scsi return code.  It is
 *	available to any driver which may want to print an error message.
 */

#ifndef  SCSIREQ
extern char *scsi_strings[];	/* driver-references definition */
#else
char	*scsi_strings[] = {	/* manager definition */
	"SCSI driver implementation error",	/* there is no error 0 */
	"SCSI bus dropped busy",		/* 1 */
	"Error during SCSI command",		/* 2 */
	"Error during SCSI status",		/* 3 */
	"Error during SCSI sense command",	/* 4 */
	"Cannot select SCSI device",		/* 5 */
	"SCSI timeout",				/* 6 */
	"id already has active request", 	/* 7 */
	"Protocol Error Processing SCSI request",	/* 8 */
	"More data than SCSI device requested",	/* 9 */
	"Less data than SCSI device requested", /* 10 */
	"SCSI extended status returned",	/* 11 */
	"SCSI Retry required",			/* 12 */
	"Caller or Scsi Manager out-of-date",	/* 13 */
	"Cancelled by Scsican",			/* 14 */
	};
#endif



/*****  scsichar - scsi device characteristics
 *
 *	int scsichar( id, value, mask)
 *	int scsidevchar ( id, value)
 *		Scsi Manager Function "scsichar" masks and sets
 *		permanent flag bits for specified id.  Returns
 *		previous flag bits.
 *
 *	flag bit definitions:
 */

int	scsichar();
#define scsidevchar(i,v) scsichar((i),(v),~0)/* map 1.0 call into 1.1 */

#define	SDC_532		1	/* unsupported 532 byte blocked */
#define	SDC_RDPOLL	2	/* slow read, do not use 4byte loop */
#define	SDC_WRPOLL	4	/* slow write, do not use 4byte loop */
#define	SDC_EXCL	8	/* Device must have exclusive use of bus */
#define	SDC_NODISC	0x10	/* never select with atn (no disconnect) */
#define	SDC_DISC	0x20	/* always select with atn (disconnect ok) */
#define SDC_TRACE      0x100	/* Record only,     bus trace */
#define SDC_TRACEPR    0x200	/* Record and Print bus trace */



/*****	scsicancel -- cancel request
 *
 *	scsicancel( id )
 *		Function scsicancel will abort and clear any request
 *		for scsi id.
 */

int	scsicancel();

struct scsitab {
    int         (*req)();               /* request entry point */
    int         (*cancel)();            /* cancel entry point */
    int         (*setchar)();           /* set-characteristics entry point */
};
extern int maxscsi;		/* max entries in scsitab[] */
extern int curscsi;		/* current number of entries in use */
#endif /* __sys_scsireq_h */
