#if !defined(__sys_swimiop_h)
#define __sys_swimiop_h

#if !defined(_NO_IDENTS) && defined(_HEAD_IDENTS)
# pragma ident "@(#)head:sys/swimiop.h	1.7 91/07/10 {Apple version 3.0 90/11/29 11:37:56}"
#endif

/*
 * Copyright 1987, 1988, 1989 Apple Computer, Inc.
 * All Rights Reserved.
 *
 * THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF APPLE COMPUTER, INC.
 * The copyright notice above does not evidence any actual or
 * intended publication of such source code.
 */

#ifndef u_char
#if defined(__MPW_C__)
# include "/:usr:include:sys:types.h"
#else
# include <sys/types.h>
#endif
#endif

/* SWIM I/O Processor Interface */

/* IOP and channel values: */
#define	SWIM_IOP	1	/* SWIM IOP */
#define	SWIM_CHAN	2	/* the SWIM driver */

/* Command code values: */

#define	cc_init		0x01	/* initialize */
#define	cc_shutdown	0x02	/* shutdown (not used) */
#define	cc_startpoll	0x03	/* start polling for insert/eject */
#define	cc_stoppoll	0x04	/* stop  " (not used) */
#define	cc_sethfstag	0x05	/* set HFS tag buffer address */
#define	cc_status	0x06	/* status */
#define	cc_eject	0x07	/* eject */
#define	cc_format	0x08	/* format */
#define	cc_fmtverify	0x09	/* format-verify */
#define	cc_write	0x0a	/* write */
#define	cc_read		0x0b	/* read */
#define	cc_rdverify	0x0c	/* read-verify */
#define	cc_cachectl	0x0d	/* cache control */
#define	cc_tagbufctl	0x0e	/* tag buffer control */
#define	cc_geticon	0x0f	/* get icon */

/* Error Codes, most of which reduce to EIO or ENXIO: */

#define	gcrOnMFMErr	-400	/* GCR (400/800K) on HD media */
#define	verErr		-84	/* verify failed */
#define	fmt2Err		-83	/* cant get enough sync during format */
#define	fmt1Err		-82	/* can't find sector 0 after track format */
#define	sectNFErr	-81	/* can't find sector */
#define	seekErr		-80	/* drive error during seek */
#define	spdAdjErr	-79	/* can't set drive speed */
#define	twoSideErr	-78	/* drive is single-sided */
#define	initIWMErr	-77	/* error during initialization */
#define	tk0badErr	-76	/* track zero is bad */
#define	cantStepErr	-75	/* drive error during step */
#define	wrUnderrun	-74	/* write underrun occurred */
#define	badDBtSlp	-73	/* bad data bitslip marks */
#define	badDCksum	-72	/* bad data checksum */
#define	noDtaMkErr	-71	/* can't find data mark */
#define	badBtSlpErr	-70	/* bad address bitslip marks */
#define	badCksmErr	-69	/* bad address-mark checksum */
#define	dataVerErr	-68	/* read-verify failed */
#define	noAdrMkErr	-67	/* can't find an address mark */
#define	noNybErr	-66	/* no nybbles? disk is probably degaussed */
#define	offLinErr	-65	/* no disk in drive */
#define	noDriveErr	-64	/* drive isn't connected */
#define	nsDrvErr	-56	/* no such drive */
#define	paramErr	-50	/* bad positioning information */
#define	wPrErr		-44	/* write protected */
#define	openErr		-23	/* already initialized */

#define	DRV_NONE	0	/* no drive */
#define	DRV_UNK		1	/* unspecified drive */
#define	DRV_400K	2	/* 400K only */
#define	DRV_800K	3	/* 400K/800K */
#define	DRV_FDHD	4	/* 400K/800K/720K/1440K FDHD */
#define	DRV_HD20	7	/* HD20 */

struct drvstatus {
    u_short	track;		/* current track */
    u_char	wprot;		/* 0x80 if write protected */
    u_char	hasdisk;	/* 1 or 2 if disk in place */
    u_char	installed;	/* 1 if drive installed, 0xff if not */
    u_char	sides;		/* 0x80 if two-sided diskette format */
    u_char	twosided;	/* 0xff if two-sided format diskette */
    u_char	newintfc;	/* 0x00 if old 400K drive, 0xff if newer */
    u_short	errors;		/* disk error count */
    struct drvinfo {		/* u_long */
	u_short	reserved;
	u_short	:4;
	u_short	external:1;	/* external drive */
	u_short	scsi:1;		/* SCSI drive */
	u_short	fixed:1;	/* fixed media */
	u_short	secondary:1;	/* secondary (not primary) drive */
	u_char	type;		/* type per DRV_XXXX types */
    } info;
    u_char	mfmdrive;	/* 0xff if FDHD drive */
    u_char	mfmdisk;	/* 0xff if 720K/1440K (MFM) format */
    u_char	mfmformat;	/* 0x00 if 720K, 0xff if 1440K format */
    u_char	diskctlr;	/* 0x00 if IWM, 0xff if SWIM */
    u_short	curformat;	/* current format type */
    u_short	allowfmt;	/* allowed format types */
    u_long	blocks;		/* number of blocks on disk */
    u_char	iconflags;	/* iconflags */
    u_char	junk1;		/* unused */
};

/* Current and allowed formats: */

#define	fmt_hd20	0x0001	/* HD20 format */
#define	fmt_400K	0x0002	/*  400K GCR */
#define	fmt_800K	0x0004	/*  800K GCR */
#define	fmt_720K	0x0008	/*  720K MFM */
#define	fmt_1440K	0x0010	/* 1440K MFM */

/* Icon Flags: */

#define	icn_media	0x01	/* have IOP supply media icon */
#define	icn_drive	0x01	/* have IOP supply drive icon */

/* Commands issued from the host CPU to the SWIM IOP: */

struct cmd_init {		/* initialize */
    u_char	code;		/* cmd code 0x01 */
    u_char	junk1;		/* unused */
    u_short	error;		/* error code */
    u_char	drives[28];	/* drive type list */
};

struct cmd_startpoll {		/* start polling for disk insert/eject */
    u_char	code;		/* cmd code 0x03 */
    u_char	junk1;		/* unused */
    u_short	error;		/* error code */
};

struct cmd_sethfstag {
    u_char	code;		/* cmd code 0x05 */
    u_char	junk1;		/* unused */
    u_short	error;		/* error code */
    caddr_t	tagbuf;		/* HFS tag buffer address */
};

struct cmd_status {
    u_char	code;		/* cmd code 0x06 */
    u_char	drive;		/* drive number */
    u_short	error;		/* error code */
    struct drvstatus status;	/* drive status */
};

struct cmd_eject {
    u_char	code;		/* cmd code 0x07 */
    u_char	drive;		/* drive number */
    u_short	error;		/* error code */
    struct drvstatus status;	/* drive status */
};

#define	fmtkind_400K	1
#define	fmtkind_800K	2
#define	fmtkind_720K	3
#define	fmtkind_1440K	1

struct cmd_format {
    u_char	code;		/* cmd code 0x08 */
    u_char	drive;		/* drive number */
    u_short	error;		/* error code */
    union {
	struct {
	    u_short	fmt;		/* format kind (fmt_xxxx) */
	    u_char	hdrbyte;	/* fmt byte for header (0=default) */
	    u_char	interleave;	/* interleave (0=default) */
	    caddr_t	databuf;	/* sector data buffer (0=default) */
	    caddr_t	tagbuf;		/* tag data buffer (0=default) */
	} f;

	struct drvstatus status;
    } p;
};

struct cmd_fmtverify {
    u_char	code;		/* cmd code 0x09 */
    u_char	drive;		/* drive number */
    u_short	error;		/* error code */
};

struct cmd_write {
    u_char	code;		/* cmd code 0x0a */
    u_char	drive;		/* drive number */
    u_short	error;		/* error code */
    caddr_t	buf;		/* buffer address */
    u_long	block;		/* starting block */
    u_long	nblks;		/* number of blocks */
    u_char	tag[12];	/* tag data */
};

struct cmd_read {
    u_char	code;		/* cmd code 0x0b */
    u_char	drive;		/* drive number */
    u_short	error;		/* error code */
    caddr_t	buf;		/* buffer address */
    u_long	block;		/* starting block */
    u_long	nblks;		/* number of blocks */
    u_char	tag[12];	/* tag data */
};

struct cmd_rw {			/* generic read/write command */
    u_char	code;		/* cmd code 0x0a (write) or 0x0b (read) */
    u_char	drive;		/* drive number */
    u_short	error;		/* error code */
    caddr_t	buf;		/* buffer address */
    u_long	block;		/* starting block */
    u_long	nblks;		/* number of blocks */
    u_char	tag[12];	/* tag data */
};

struct cmd_rdverify {
    u_char	code;		/* cmd code 0x0c */
    u_char	drive;		/* drive number */
    u_short	error;		/* error code */
    caddr_t	buf;		/* buffer address */
    u_long	block;		/* starting block */
    u_long	nblks;		/* number of blocks */
    u_char	tag[12];	/* tag data */
};

struct cmd_cachectl {
    u_char	code;		/* cmd code 0x0d */
    u_char	unused;
    u_short	error;		/* error code */
    u_char	enable;		/* nonzero to enable */
    u_char	install;	/* +1 to install, -1 to remove, 0 neither */
};

struct cmd_tagbufctl {
    u_char	code;		/* cmd code 0x0e */
    u_char	unused;
    u_short	error;		/* error code */
    caddr_t	buf;		/* buffer address (0 to disable) */
};

struct cmd_geticon {
    u_char	code;		/* cmd code 0x0f */
    u_char	drive;		/* drive number */
    u_short	error;		/* error code */
    caddr_t	buf;		/* buffer address */
    u_short	kind;		/* 0=media, 1=drive icon kind */
    u_short	unused;
    u_short	maxbytes;	/* max byte count */
};

/* Messages from the SWIM IOP to the host CPU: */

struct msg_insert {		/* disk inserted */
    u_char	code;		/* cmd code 0x01 */
    u_char	drive;		/* drive number */
    u_short	error;		/* error code */
    struct drvstatus status;	/* status */
};

struct msg_eject {		/* eject button pressed */
    u_char	code;		/* cmd code 0x02 */
    u_char	drive;		/* drive number */
    u_short	error;		/* error code */
    struct drvstatus status;	/* status */
};

struct msg_status {		/* disk status changed */
    u_char	code;		/* cmd code 0x03 */
    u_char	drive;		/* drive number */
    u_short	error;		/* error code */
    struct drvstatus status;	/* status */
};

#endif /* __sys_swimiop_h */
